import { resolve, dirname } from "node:path";
import { renderPlop } from "./render.js";
import { fileURLToPath } from "node:url";

const __dirname = dirname(fileURLToPath(import.meta.url));

test("should report a missing plopfile when not copied", async () => {
  await expect(() => renderPlop()).rejects.toMatchInlineSnapshot(
    /\[PLOP\] No plopfile found/,
    `Object {}`
  );
});

test("should show help information on help flag", async () => {
  const { findByText } = await renderPlop(["--help"]);
  const { stdoutStr } = await findByText("Usage:");
  expect(stdoutStr).toMatchSnapshot();
});

test("should show version on version flag", async () => {
  const { findByText } = await renderPlop(["--version"]);
  expect(await findByText(/^[\w\.-]+$/)).toBeTruthy();
});

test("should show version on v flag", async () => {
  const { findByText } = await renderPlop(["-v"]);
  expect(await findByText(/^[\w\.-]+$/)).toBeTruthy();
});

test("should display inquirer prompts", async () => {
  const { findByText, userEvent } = await renderPlop([], {
    cwd: resolve(__dirname, "./examples/prompt-only"),
  });
  expect(await findByText("What is your name?")).toBeTruthy();
  userEvent.keyboard("Joe");
  expect(await findByText("Joe")).toBeTruthy();
  userEvent.keyboard("[Enter]");
});

test("Should handle generator prompt", async () => {
  const { findByText, clear, userEvent } = await renderPlop([""], {
    cwd: resolve(__dirname, "./examples/javascript"),
  });

  await findByText("Please choose a generator");

  clear();
  userEvent.keyboard("[ArrowUp]");
  userEvent.keyboard("[ArrowDown]");
  userEvent.keyboard("[Enter]");

  expect(await findByText("this is a test")).toBeTruthy();
});

test("Should bypass generator prompt", async () => {
  const { findByText } = await renderPlop(["test"], {
    cwd: resolve(__dirname, "./examples/javascript"),
  });

  expect(await findByText("What is your name?")).toBeTruthy();
});

test("Should bypass prompt by input", async () => {
  const { queryByText, findByText } = await renderPlop(["Frank"], {
    cwd: resolve(__dirname, "./examples/prompt-only"),
  });

  expect(await queryByText("What is your name?")).toBeFalsy();
  expect(await findByText("What pizza toppings do you like?")).toBeTruthy();
});

test("Should bypass prompt by input placeholder", async () => {
  const { queryByText, findByText, userEvent } = await renderPlop(
    ["_", "Cheese"],
    {
      cwd: resolve(__dirname, "./examples/prompt-only"),
    }
  );

  expect(await findByText("What is your name?")).toBeTruthy();
  userEvent.keyboard("[Enter]");
  expect(await queryByText("What pizza toppings do you like?")).toBeFalsy();
});

test("Should bypass prompt by name", async () => {
  const { queryByText, findByText } = await renderPlop(
    ["--", "--name", "Frank"],
    {
      cwd: resolve(__dirname, "./examples/prompt-only"),
    }
  );

  expect(await queryByText("What is your name?")).toBeFalsy();
  expect(await findByText("What pizza toppings do you like?")).toBeTruthy();
});

test("Should allow for empty string bypassing", async () => {
  const { queryByText, findByText } = await renderPlop(["--", "--name", `""`], {
    cwd: resolve(__dirname, "./examples/prompt-only"),
  });

  expect(await queryByText("What is your name?")).toBeFalsy();
  expect(await findByText("What pizza toppings do you like?")).toBeTruthy();
});

test.todo("Dynamic actions");
